#include "capteur.h"

Siplec::Siplec(int nb_hist)
{
  NbHist = nb_hist;
  history = new double[NbHist];
  timestamp = new char[NbHist*5];
  unite = "Celsius";
}
double Siplec::get_value()
{
  return 0;
}
void Siplec::init(int value) {}
void Siplec::set_value(double value, int pos)
{
  history[pos] = value;
}

void Siplec::set_timestamp(char* timestamp_source, int pos){
  strncpy(&timestamp[pos*5], timestamp_source, 5);
}
void Siplec::get_timestamp(char* timestamp_dest, int pos){
  strncpy(timestamp_dest, &timestamp[pos*5], 5);
}

double Siplec::get_hist(int pos)
{
  return history[pos];
}

class GP2Y10: public capteur
{
  public:
    GP2Y10(int sensor, int led, int nb_hist)
    {
      pin_sensor = sensor;
      pin_led = led;
      NbHist = nb_hist;
      history = new double[NbHist];
      unite = "1/10mg/m3";
    }
    void init(int value)
    {
      pinMode(pin_led, OUTPUT);
    };
    double get_value()
    {
      for ( int i = NbHist; i--; i > 0)
      {
        history[i] = history[i - 1];
      }
      digitalWrite(pin_led, LOW);
      delayMicroseconds(300);
      digitalWrite(pin_led, HIGH);
      history[0] = analogRead(pin_sensor);
      history[0] = history[0] - 20;
      if (history[0] < 0)
      {
        history[0] = 0;
      }
      history[0] = history[0] / 100;
      return history[0];
    }
  private:
    int pin_sensor, pin_led;
};

class MQ135: public capteur
{
  public:
    MQ135(int pin, int nb_hist)
    {
      channel = pin;
      NbHist = nb_hist;
      history = new double[NbHist];
      unite = "ppm";
    }
    double get_value()
    {
      for ( int i = NbHist; i--; i > 0)
      {
        history[i] = history[i - 1];
      }
      history[0] = analogRead(channel);
      history[0] = history[0] / 5;
      return history[0];
    }
    void init(int value) {};
    double get_hist(int pos)
    {
      return history[pos];
    }
  private:
    int channel;
};

class t_diode: public capteur
{
  public:
    t_diode(int pin, double a, double b, int nb_hist)
    {
      channel = pin;
      A = a;
      B = b;
      NbHist = nb_hist;
      history = new double[NbHist];
      unite = "celcius";
    }
    double get_value()
    {
      for ( int i = NbHist; i--; i > 0)
      {
        history[i] = history[i - 1];
      }
      history[0] = analogRead(channel);
      delay(50);
      history[0] += analogRead(channel);
      delay(50);
      history[0] += analogRead(channel);
      history[0] = history[0] / 3;
      history[0] = A * history[0] + B ;
      return history[0];
    }
    void init(int value) {};
    double get_hist(int pos)
    {
      return history[pos];
    }
  private:
    int channel;
    double A, B;
};
