#include "siplec433.h"

#define mean_bit_time 600    //durée pour définir bit à 1 ou 0
#define max_bit_time  3000    //durée maxi d'un bit
#define min_bit_time  1000    //durée mini d'un bit
#define temperature_index  15  //emplacement du 1er bit de température
#define num_sonde_index 8      //emplacement bit de 2eme sonde
#define bit_signe_index 13           //emplacement du bit de signe
#define length_max_trame  100
#define identifiant2 "11111111100001100"
#define identifiant1 "11111111100001101"
#define led  5
//----------variables
static long temps, last_bit_time, timming;
static char trame[length_max_trame];
static int trame_ptr,i;
static int rc_b, id_ok, var, var1, nb_sonde;;
static int last_temp;
static int new_temp;

void init_433(void){
  rc_b = 0;
  trame_ptr = 0;
  id_ok = 0;
  last_temp = 0;
  new_temp = 0;
  temps = micros();
  pinMode(pin_433, INPUT);
  attachInterrupt(digitalPinToInterrupt(pin_433),INT_433,CHANGE);
  pinMode(led, OUTPUT);
  digitalWrite(led, HIGH);
  Serial.println("start433");
}

void INT_433(void){
  traite_433_2(digitalRead(pin_433));
}

void traite_433_2(int pin_433_value){
  last_bit_time = micros() - temps;  
  temps = micros();
  //front montant
  if (pin_433_value){
    if(rc_b){
      rc_b = 0;
      //detection fin de trame
      if (last_bit_time > max_bit_time){
      //  trame_ptr--;
      //  trame[trame_ptr++] = ' ';
      }
      else{
        trame[trame_ptr++] = 48 + (last_bit_time < mean_bit_time);
      }
    }
  }
  //front descendant
  else{
    if((last_bit_time > min_bit_time) & (last_bit_time < max_bit_time)){
      rc_b = 1;
    }
  }
  //detection identifiant
  if( !id_ok & ((strstr(trame, identifiant1) != NULL)|(strstr(trame, identifiant2) != NULL) )){
    id_ok = 1;
    digitalWrite(led,!digitalRead(led));
  }
  else if(!id_ok ){ //buffer circulaire
    if (trame_ptr > sizeof(identifiant1) -2){
      for(i=1; i < trame_ptr; i++){
        trame[i-1] = trame[i];
      }
      trame_ptr = sizeof(identifiant1) -2; 
    }
    //id_ok = 0;
  }
    // detection fin transmission
  if(trame_ptr == length_max_trame - 1){
    trame[trame_ptr] = '\0';
    traite_trame(trame);
    if (check_sum){
      rc_b = 0;
      var = 0;
      var1 = 0;
      //dizaine
      for(i = temperature_index; i < temperature_index + 4; i++){
        var1 += (trame[i] - 48);
        var1 = var1 * 2;
      }
      var = var1 *50;
      //unite
      var1 = 0;
      for(i = temperature_index + 4; i < temperature_index + 8; i++){
        var1 += (trame[i] - 48);
        var1 = var1 * 2;
      }
      var += var1 *5;
      //dixieme
      var1 = 0;
      for(i = temperature_index + 8; i < temperature_index + 12; i++){
        var1 += (trame[i] - 48);
        var1 = var1 * 2;
      }
      var += var1 /2;
      if(trame[bit_signe_index]-48){
        var = -var;
      }
      if ((var > -200) & (var < 600)){  //température "valide"
        last_temp = var;
        new_temp = 2-(trame[num_sonde_index]-48);
      }
      for (i=0; i < trame_ptr; i++){
        trame[i] = '\0';
      }
      id_ok = 0;
      trame_ptr = 0;  
    }
    else{
      //checksum pas OK
      rc_b = 0;
      id_ok = 0;
      trame_ptr = 0;
    }
  }  
}

void traite_trame(char* trame){
  //supprime les '01' qui suivent un '1'
  for(int i = 0; i < length_max_trame; i++){
    if(trame[i] == '1'){
      for(int y=i+1; y < length_max_trame -2; y++){
        trame[y] = trame[y+2];
      }
    }
  }
}

int check_sum(char* trame){
  //renvoie true si la trame est OK
  return true; // comment marche le checksum ??  
}

int get_temp(void){
  new_temp = 0;
  return last_temp;
}

int is_new_temp(void){
  //renvoie le n° de la sonde ou 0
  return new_temp;
}

